@@ -11,21 +11,14 @@ mod_Predictive_Power_ui <- function(id){
   ns <- NS(id)
   
   # PREDICTIVE POWER PAGE ---------------------------------------------------------------------------------------------------
-  
-  code.power.num <- list(h3(labelInput("codigo")), hr(),
-                         aceEditor(ns("fieldCodePoderNum"), mode = "r", theme = "monokai",
-                                   value = "", height = "7vh", readOnly = F, autoComplete = "enabled"))
-  
-  
-  tabs.power.num <- tabsOptions(buttons = list(icon("terminal")), widths = 100, heights = 55,
-                                tabs.content = list(code.power.num))
-  
+
+
   power.plot.pairs <- tabPanel(title = labelInput("pares"), value = "predpares",
                                withLoader(plotOutput(ns('plot_pairs_poder'), height = "75vh"),
                                           type = "html", loader = "loader4"))
   
   pagina.poder <- tabItem(tabName = "poderPred",
-                          tabBoxPrmdt(id = ns("BoxPodPred"), opciones = tabs.power.num,
+                          tabBoxPrmdt(id = ns("BoxPodPred"), 
                                  power.plot.pairs))
   
   
@@ -37,18 +30,18 @@ mod_Predictive_Power_ui <- function(id){
 #' Predictive_Power Server Function
 #'
 #' @noRd 
-mod_Predictive_Power_server <- function(input, output, session, updateData){
+mod_Predictive_Power_server <- function(input, output, session, updateData, codedioma){
   ns <- session$ns
   # Show the graph of numerical predictive power
   output$plot_pairs_poder <- renderPlot({
     tryCatch({
       updateData$datos.aprendizaje
-      updateAceEditor(session, "fieldCodePoderNum", value = "pairs_power(datos)")
+      isolate(codedioma$code <- append(codedioma$code, paste0("### pares\n", "pairs_power(datos)")))
       if (ncol(var.numericas(updateData$datos)) >= 2) {
         if(ncol(var.numericas(updateData$datos)) <= 25){
           pairs_power(updateData$datos, decimals = updateData$decimals)
         }else{
-          showNotification(tr("bigPlot",updateData$idioma), duration = 10, type = "message")
+          showNotification(tr("bigPlot",codedioma$idioma), duration = 10, type = "message")
           NULL
         }
       }else{
