@@ -11,93 +11,52 @@ mod_dimension_reduction_ui <- function(id){
   
   ns <- NS(id)
   
-  rd.options  <- list(options.run(ns("runRd")), tags$hr(style = "margin-top: 0px;"),
-                      fluidRow(column(selectInput(inputId = ns("modo.rd"), label = labelInput("selectAlg"),selected = 0,
-                                                  choices = list("ACP" = 0, "MCP" = 1)),width = 5),
-                               column(width=5,radioSwitch(id = ns("switch_scale_rd"), label = "escal",names = c("si", "no")))),
-                      fluidRow(column(id = ns("colManualCom"),width = 5, 
-                                      numericInput(ns("ncomp.rd"), labelInput("ncomp"),value = 2, min = 1, width = "100%")),
-                               column(width = 5, radioSwitch(id = ns("permitir_ncomp"), label = "",
-                                                             names = c("manual", "automatico"), val.def = FALSE))))
-  
-  
-  rd.code.config <- list(h3(labelInput("codigo")), hr(style = "margin-top: 0px;"),
-                         codigo.monokai(ns("fieldCodeRd"), height = "8vh"))
-  
-  
-  rd.code   <- list(fluidRow(column(width = 9,h3(labelInput("codigo")))),
-                    hr(style = "margin-top: 0px;"),
-                    conditionalPanel("input.BoxRd == 'tabRdRMSE'",
-                                     codigo.monokai(ns("fieldCodeRdRMSE"), height = "7vh"), ns = ns),
-                    conditionalPanel("input.BoxRd == 'tabRdPlotPred'",
-                                     codigo.monokai(ns("fieldCodeRdPlotPred"), height = "7vh"), ns = ns),
-                    conditionalPanel("input.BoxRd == 'tabRdPlotVarPred'",
-                                     codigo.monokai(ns("fieldCodeRdPlotVarPred"), height = "7vh"), ns = ns),
-                    conditionalPanel("input.BoxRd == 'tabRdPred'",
-                                     codigo.monokai(ns("fieldCodeRdPred"), height = "7vh"), ns = ns),
-                    conditionalPanel("input.BoxRd == 'tabRdDisp'",
-                                     codigo.monokai(ns("fieldCodeRdDisp"), height = "7vh"), ns = ns),
-                    conditionalPanel("input.BoxRd == 'tabRdIndex'",
-                                     codigo.monokai(ns("fieldCodeRdIG"), height = "7vh"), ns = ns))
-  
-  
-  
-  tabs.options.generate <- tabsOptions(buttons = list(icon("cog"), icon("code")), widths = c(50,100), heights = c(80,70),
-                                       tabs.content = list(rd.options,rd.code.config))
-  
-  tabs.options.Nogenerate <- tabsOptions(buttons = list(icon("code")), widths = c(100), heights = c(70),
-                                         tabs.content = list(rd.code))
-  
-  tabs.options <- list(conditionalPanel("input.BoxRd == 'tabRdModelo'",tabs.options.generate,ns = ns),
-                       conditionalPanel("input.BoxRd != 'tabRdModelo'",tabs.options.Nogenerate,ns = ns))
-  
-  
-  generate.rd.panel <- tabPanel(title = labelInput("generatem"),value = "tabRdModelo",
-                                withLoader(verbatimTextOutput(ns("txtRd")),type = "html", loader = "loader4"))
-  
-  rmse.rd.panel <- tabPanel(title = labelInput("RMSE"),value = "tabRdRMSE",
-                            withLoader(echarts4rOutput(ns('plot_rd_rmse'),height = "75vh"),type = "html", loader = "loader4"))
-  
-  plot.pred.rd.panel <- tabPanel(title = labelInput("RdPred"), value = "tabRdPlotPred",
-                                 withLoader(echarts4rOutput(ns('plot_rd_pred'),height = "75vh"),type = "html", loader = "loader4"))
-  
-  panel.plot.var.pred.rd <- tabPanel(title = labelInput("RdVarPred"), value = "tabRdPlotVarPred",
-                                     withLoader(echarts4rOutput(ns('plot_rd_var_pred'),height = "75vh"),type = "html", loader = "loader4"))
-  
-  prediction.rd.panel <- tabPanel(title = labelInput("predm"), value = "tabRdPred",
-                                  withLoader(DT::dataTableOutput(ns("rdPrediTable")),type = "html", loader = "loader4"))
-  
-  disp.rd.panel <- tabPanel(title = labelInput("dispersion"), value = "tabRdDisp",
-                            withLoader(echarts4rOutput(ns('plot_rd_disp'),height = "75vh"),type = "html", loader = "loader4"))
-  
-  general.index.rd.panel <- tabPanel(title = labelInput("indices"), value = "tabRdIndex",
-                                     br(),
-                                     fluidRow(withLoader(tableOutput(ns('indexdfrd')),type = "html", loader = "loader4")),
-                                     br(),
-                                     fluidRow(column(width = 12, align="center", tags$h3(labelInput("resumenVarPre")))),
-                                     br(),
-                                     fluidRow(withLoader(tableOutput(ns('indexdfrd2')),type = "html", loader = "loader4")))
-  
-  page.rd <- tabItem(tabName = "rd",
-                     tabBoxPrmdt(id = ns("BoxRd"), opciones = tabs.options,
-                            generate.rd.panel,
-                            rmse.rd.panel,
-                            plot.pred.rd.panel,
-                            panel.plot.var.pred.rd,
-                            prediction.rd.panel,
-                            disp.rd.panel,
-                            general.index.rd.panel))
-  
+  tabs.options <- list(
+    conditionalPanel("input.BoxRd == 'tabRdModelo'", tabsOptions(widths = c(100), heights = c(80),
+        tabs.content = list(list(options.run(ns("runRd")), tags$hr(style = "margin-top: 0px;"),
+                                 fluidRow(col_6(selectInput(inputId = ns("modo.rd"), label = labelInput("selectAlg"),selected = 0,
+                                                             choices = list("ACP" = 0, "MCP" = 1))),
+                                          col_6(radioSwitch(id = ns("switch_scale_rd"), label = "escal",names = c("si", "no")))),
+                                 fluidRow(col_6(id = ns("colManualCom"), 
+                                                 numericInput(ns("ncomp.rd"), labelInput("ncomp"),value = 2, min = 1, width = "100%")),
+                                          col_6(radioSwitch(id = ns("permitir_ncomp"), label = "",
+                                                                        names = c("manual", "automatico"), val.def = FALSE)))))),ns = ns))
   
+
   tagList(
-    page.rd
+    tabBoxPrmdt(id = ns("BoxRd"), opciones = tabs.options,
+                tabPanel(title = labelInput("generatem"),value = "tabRdModelo",
+                         withLoader(verbatimTextOutput(ns("txtRd")),
+                                    type = "html", loader = "loader4")),
+                tabPanel(title = labelInput("RMSE"),value = "tabRdRMSE",
+                         withLoader(echarts4rOutput(ns('plot_rd_rmse'),height = "75vh"),
+                                    type = "html", loader = "loader4")),
+                tabPanel(title = labelInput("RdPred"), value = "tabRdPlotPred",
+                         withLoader(echarts4rOutput(ns('plot_rd_pred'),height = "75vh"),
+                                    type = "html", loader = "loader4")),
+                tabPanel(title = labelInput("RdVarPred"), value = "tabRdPlotVarPred",
+                         withLoader(echarts4rOutput(ns('plot_rd_var_pred'),height = "75vh"),
+                                    type = "html", loader = "loader4")),
+                tabPanel(title = labelInput("predm"), value = "tabRdPred",
+                         withLoader(DT::dataTableOutput(ns("rdPrediTable")),
+                                    type = "html", loader = "loader4")),
+                tabPanel(title = labelInput("dispersion"), value = "tabRdDisp",
+                         withLoader(echarts4rOutput(ns('plot_rd_disp'),height = "75vh"),
+                                    type = "html", loader = "loader4")),
+                tabPanel(title = labelInput("indices"), value = "tabRdIndex",
+                         br(),
+                         div(withLoader(tableOutput(ns('indexdfrd')),type = "html", loader = "loader4")),
+                         br(),
+                         div(col_12(align="center", tags$h3(labelInput("resumenVarPre")))),
+                         br(),
+                         div(withLoader(tableOutput(ns('indexdfrd2')),type = "html", loader = "loader4"))))
   )
 }
 
 #' dimension_reduction Server Function
 #'
 #' @noRd 
-mod_dimension_reduction_server <- function(input, output, session,updateData, modelos){
+mod_dimension_reduction_server <- function(input, output, session,updateData, modelos, codedioma, modelos2){
   ns <- session$ns
   
   nombreBase <- "modelo.rd."
@@ -105,6 +64,9 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
   
   ncomp <- NULL
   
+  observeEvent(c(updateData$datos), {
+    modelos2$rd = list(n = 0, mcs = vector(mode = "list", length = 10))
+  })
   return.rd.default.values <- function(){
     updateSelectInput(session,"modo.rd",selected = 0)
     updateRadioSwitch(session,"switch_scale_rd","TRUE")
@@ -120,11 +82,6 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
     return.rd.default.values()
   })
   
-  observeEvent(input$runRd, {
-    if (validate_data(updateData, idioma = updateData$idioma)) { # Si se tiene los datos entonces :
-      rd_full()
-    }
-  })
   
   
   # Habilitada o deshabilitada el número de componenetes 
@@ -137,16 +94,16 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
   })
   
   
-  # Execute model, prediction and indices
-  rd_full <- function(){
+  #Update model tab
+  output$txtRd <- renderPrint({
+    input$runRd
     tryCatch({
-      shinyjs::runjs(code = "generating_model = true")
-      
+      codigo.rd()
       isolate({
         datos.aprendizaje <- updateData$datos.aprendizaje
-        datos.prueba <- updateData$datos.prueba
+        datos.prueba      <- updateData$datos.prueba
         variable.predecir <- updateData$variable.predecir
-        scale <- as.logical(input$switch_scale_rd)
+        scale   <- as.logical(input$switch_scale_rd)
         modo.rd <- input$modo.rd
         
         ncomp <<- NULL
@@ -165,36 +122,33 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
         ncomp <<- modelo.rd$optimal.n.comp
         updateNumericInput(session,"ncomp.rd", value = ncomp)
       }
-      updateAceEditor(session, "fieldCodeRd", value = codeRd(variable.predecir,modo.rd, scale))
-      
+
       #Prediccion
-      prediccion.rd <- rd_prediction(modelo.rd, datos.prueba, ncomp)
-      updateAceEditor(session, "fieldCodeRdPred", value = codeRdPred(nombreModelo, ncomp))
-      
+      prediccion.rd <- predict(modelo.rd, datos.prueba, ncomp = ncomp)
+
       #Indices
       indices.rd <- general_indices(datos.prueba[,variable.predecir], prediccion.rd)
-      updateAceEditor(session, "fieldCodeRdIG", value = codeRdIG(variable.predecir))
-      
+
       #isolamos para que no entre en un ciclo en el primer renderPrint
-      isolate(modelos$rd[[nombreModelo]] <- list(modelo = modelo.rd, prediccion = prediccion.rd, indices = indices.rd, 
-                                                 id = rd_type(modo.rd)))
-    }, error = function(e){
-      isolate(modelos$rd[[nombreModelo]] <- NULL)
-      showNotification(paste0("Error (RD-00) : ",e), duration = 10, type = "error")
-    },
-    finally = {shinyjs::runjs(code = "generating_model = false")})
-  }
-  
-  
-  #Update model tab
-  output$txtRd <- renderPrint({
-    tryCatch({
+
+      isolate({
+        modelos$rd[[nombreModelo]] <- list(modelo = modelo.rd, prediccion = prediccion.rd, indices = indices.rd, 
+                                            id = rd_type(modo.rd))
+        modelos2$rd$n <- modelos2$rd$n + 1
+        modelos2$rd$mcs[modelos2$rd$n] <- list(indices.rd)
+        if(modelos2$rd$n > 9)
+          modelos2$rd$n <- 0
+        
+      })
+      
       if(!is.null(modelos$rd[[nombreModelo]])){
         modelo.rd <- modelos$rd[[nombreModelo]]$modelo
-        print(summary(modelo.rd))
+        print(modelo.rd)
       }
       else{NULL}
+      
     }, error = function(e){
+      isolate(modelos$rd[[nombreModelo]] <- NULL)
       showNotification(paste0("Error (RD-01) : ",e), duration = 10, type = "error")
       NULL
     })
@@ -205,11 +159,13 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
     tryCatch({
       if(!is.null(modelos$rd[[nombreModelo]])){
         modelo.rd <- modelos$rd[[nombreModelo]]$modelo
-        idioma <- updateData$idioma
+        idioma <- codedioma$idioma
         
         # Actualizar el código en el AceEditor
         codigo <- paste0("plot_RMSE(",nombreModelo, ", ncomp = ", ncomp ,")")
-        updateAceEditor(session, "fieldCodeRdRMSE", value = codigo)
+        cod    <- paste0("### Rmse\n",codigo, "\n")
+        
+        isolate(codedioma$code <- append(codedioma$code, cod))
         
         titulos <- c(
           tr("RMSEComp", idioma),
@@ -233,11 +189,13 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
       if(!is.null(modelos$rd[[nombreModelo]])){
         
         modelo.rd <- modelos$rd[[nombreModelo]]$modelo
-        idioma <- updateData$idioma
+        idioma <- codedioma$idioma
         
         # Se actualiza el codigo
         codigo <- paste0("plot_pred_rd(",nombreModelo, ", ncomp = ", ncomp ,")")
-        updateAceEditor(session, "fieldCodeRdPlotPred", value = codigo)
+        cod    <- paste0("### RdPred\n",codigo, "\n")
+        
+        isolate(codedioma$code <- append(codedioma$code, cod))
         
         titulos <- c(
           tr("RdPred", idioma),
@@ -260,11 +218,13 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
     tryCatch({
       if(!is.null(modelos$rd[[nombreModelo]])){
         modelo.rd <- modelos$rd[[nombreModelo]]$modelo
-        idioma <- updateData$idioma
+        idioma <- codedioma$idioma
         
         # Se actualiza el codigo del plot
         codigo <- paste0("plot_var_pred_rd(",nombreModelo, ", ncomp = ", ncomp ,")")
-        updateAceEditor(session, "fieldCodeRdPlotVarPred", value = codigo)
+        cod    <- paste0("### RdVarPred\n",codigo, "\n")
+        
+        isolate(codedioma$code <- append(codedioma$code, cod))
         
         titulos <- c(
           tr("RdVarPred", idioma),
@@ -291,7 +251,7 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
           datos.prueba <- updateData$datos.prueba
           real.val <- datos.prueba[updateData$variable.predecir]
         })
-        tb_predic(real.val, prediccion.rd, updateData$decimals, updateData$idioma)
+        tb_predic(real.val, prediccion.rd, updateData$decimals, codedioma$idioma)
       }
       else{NULL}
       
@@ -315,11 +275,13 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
           variable.predecir <- updateData$variable.predecir
           modo.rd <- input$modo.rd
         })
-        idioma <- updateData$idioma
+        idioma <- codedioma$idioma
         
         model.name <- paste0(tr("rd", idioma), "-", rd_type(modo.rd))
         codigo <- disp_models(nombreModelo, model.name, variable.predecir)
-        updateAceEditor(session, "fieldCodeRdDisp", value = codigo)
+        cod    <- paste0("### docdisp\n",codigo, "\n")
+        
+        isolate(codedioma$code <- append(codedioma$code, cod))
         
         titulos <- c(
           tr("predvsreal", idioma),
@@ -342,7 +304,7 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
   output$indexdfrd <- renderTable({
     tryCatch({
       if(!is.null(modelos$rd[[nombreModelo]])){
-        idioma <- updateData$idioma
+        idioma <- codedioma$idioma
         indices.rd <- modelos$rd[[nombreModelo]]$indices
         tabla.indicesPrecision(indices.rd, updateData$decimals, idioma)
       }
@@ -358,10 +320,12 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
   
   output$indexdfrd2 <- renderTable({
     tryCatch({
-      if(!is.null(modelos$rd[[nombreModelo]])& !is.null(updateData$summary.var.pred)){
-        idioma <- updateData$idioma
+      if(!is.null(modelos$rd[[nombreModelo]])){
+        idioma <- codedioma$idioma
         decimals <- updateData$decimals
-        tabla.varpred.summary(updateData$summary.var.pred, decimals, idioma)
+        tabla.varpred.summary(summary_indices(updateData$datos.prueba[,updateData$variable.predecir]),
+                              decimals, 
+                              idioma)
       }
       else{NULL}
     }
@@ -372,6 +336,37 @@ mod_dimension_reduction_server <- function(input, output, session,updateData, mo
   },striped = TRUE, bordered = TRUE, spacing = 'l', 
   width = '100%', align = 'c')
   
+  
+  # Execute model, prediction and indices
+  codigo.rd <- function(){
+    
+    tryCatch({
+      
+      isolate({
+        variable.predecir <- updateData$variable.predecir
+        scale   <- as.logical(input$switch_scale_rd)
+        modo.rd <- input$modo.rd
+        ncomp   <- input$ncomp.rd
+      })
+      
+      #Model generate
+      codigo <- codeRd(variable.predecir,modo.rd, scale)
+      cod    <- paste0("### RD\n", codigo)
+      
+      #Prediccion
+      codigo <- codigo.prediccion("rd")
+      cod    <- paste0(cod, codigo)
+      #Indices
+      codigo <- codigo.IG(model.name = "rd", variable.pr = variable.predecir)
+      cod    <- paste0(cod, codigo)
+      
+      isolate(codedioma$code <- append(codedioma$code, cod))
+      
+    }, error = function(e){
+      showNotification(paste0("Error (RD-00) : ",e), duration = 10, type = "error")
+    })
+  }
+  
 }
 
 ## To be copied in the UI
