@@ -15,30 +15,11 @@ mod_regression_trees_ui <- function(id){
                      fluidRow(column(numericInput(ns("minsplit.dt"), labelInput("minsplit"), 20, width = "100%",min = 1), width = 5),
                               column(numericInput(ns("maxdepth.dt"), labelInput("maxdepth"), 15, width = "100%",min = 0, max = 30, step = 1),width = 5)))
   
-  dt.code.config <- list(h3(labelInput("codigo")), hr(style = "margin-top: 0px;"),
-                         codigo.monokai(ns("fieldCodeDt"), height = "7vh"))
   
-  dt.code <- list(h3(labelInput("codigo")), hr(style = "margin-top: 0px;"),
-                  conditionalPanel("input.BoxDt == 'tabDtPlot'",
-                                   codigo.monokai(ns("fieldCodeDtPlot"), height = "7vh"),ns = ns),
-                  conditionalPanel("input.BoxDt == 'tabDtPred'",
-                                   codigo.monokai(ns("fieldCodeDtPred"), height = "7vh"),ns = ns),
-                  conditionalPanel("input.BoxDt == 'tabDtDisp'",
-                                   codigo.monokai(ns("fieldCodeDtDisp"), height = "7vh"),ns = ns),
-                  conditionalPanel("input.BoxDt == 'tabDtIndex'",
-                                   codigo.monokai(ns("fieldCodeDtIG"), height = "7vh"),ns = ns),
-                  conditionalPanel("input.BoxDt == 'tabDtReglas'",
-                                   codigo.monokai(ns("fieldCodeDtRule"), height = "7vh"),ns = ns))
+  tabs.options.generate <- tabsOptions( widths = c(100), heights = c(70),
+                                       tabs.content = list(dt.options))
   
-  
-  tabs.options.generate <- tabsOptions(buttons = list(icon("cog"), icon("code")), widths = c(50,100), heights = c(70,70),
-                                       tabs.content = list(dt.options,dt.code.config))
-  
-  tabs.options.Nogenerate <- tabsOptions(buttons = list(icon("code")), widths = c(100), heights = c(70),
-                                         tabs.content = list(dt.code))
-  
-  tabs.options <- list(conditionalPanel("input.BoxDt == 'tabDtModelo'",tabs.options.generate,ns = ns),
-                       conditionalPanel("input.BoxDt != 'tabDtModelo'",tabs.options.Nogenerate,ns = ns))
+  tabs.options <- list(conditionalPanel("input.BoxDt == 'tabDtModelo'",tabs.options.generate,ns = ns))
   
   generate.dt.panel <- tabPanel(title = labelInput("generatem"), value = "tabDtModelo",
                                 withLoader(verbatimTextOutput(ns("txtDt")),type = "html", loader = "loader4"))
@@ -54,11 +35,11 @@ mod_regression_trees_ui <- function(id){
   
   general.index.dt.panel <- tabPanel(title = labelInput("indices"),value = "tabDtIndex",
                                      br(),
-                                     fluidRow(withLoader(tableOutput(ns('indexdfdt')),type = "html", loader = "loader4")),
+                                     div(withLoader(tableOutput(ns('indexdfdt')),type = "html", loader = "loader4")),
                                      br(),
-                                     fluidRow(column(width = 12, align="center", tags$h3(labelInput("resumenVarPre")))),
+                                     div(column(width = 12, align="center", tags$h3(labelInput("resumenVarPre")))),
                                      br(),
-                                     fluidRow(withLoader(tableOutput(ns('indexdfdt2')),type = "html", loader = "loader4")))
+                                     div(withLoader(tableOutput(ns('indexdfdt2')),type = "html", loader = "loader4")))
   
   rules.dt.panel <- tabPanel(title = labelInput("reglas"),value = "tabDtReglas",
                              withLoader(verbatimTextOutput(ns("rulesDt")),type = "html", loader = "loader4"))
@@ -80,7 +61,7 @@ mod_regression_trees_ui <- function(id){
 #' regression_trees Server Function
 #'
 #' @noRd 
-mod_regression_trees_server <- function(input, output, session,updateData, modelos){
+mod_regression_trees_server <- function(input, output, session,updateData, modelos, codedioma, modelos2){
   ns <- session$ns
   
   nombreModelo <- "modelo.dt"
@@ -90,26 +71,22 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
     updateNumericInput(session,inputId = "maxdepth.dt", value = 15)
   }
   
+  observeEvent(c(updateData$datos), {
+    modelos2$dt = list(n = 0, mcs = vector(mode = "list", length = 10))
+  })
   observeEvent(updateData$datos.aprendizaje,{
     return.dt.default.values()
   })
   
   
-  #  When the dt model is generated
-  observeEvent(input$runDt, {
-    if (validate_data(updateData, idioma = updateData$idioma)) { # Si se tiene los datos entonces :
-      dt_full()
-    }
-  })
-  
-  # Execute model, prediction and indices
-  dt_full <- function() {
+  #Update model tab
+  output$txtDt <- renderPrint({
+    input$runDt
     tryCatch({
-      shinyjs::runjs(code = "generating_model = true")
-      
+      codigo.dt()
       isolate({
         datos.aprendizaje <- updateData$datos.aprendizaje
-        datos.prueba <- updateData$datos.prueba
+        datos.prueba      <- updateData$datos.prueba
         variable.predecir <- updateData$variable.predecir
         ms <- input$minsplit.dt
         md <- input$maxdepth.dt
@@ -118,43 +95,46 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
       minsplit <- ifelse(!is.numeric(ms), 20, ms)
       maxdepth <- ifelse(!is.numeric(md), 15, md)
       
+      var    <- as.formula(paste0(variable.predecir, "~."))
+      
       # Model Generate
-      modelo.dt <- dt_model(datos.aprendizaje, variable.predecir,
-                            minsplit = minsplit,
-                            maxdepth = maxdepth)
-      updateAceEditor(session, "fieldCodeDt", value = codeDt(variable.predecir,minsplit,maxdepth))
-      
+      modelo.dt <- train.rpart(var, data = datos.aprendizaje,
+                               control = rpart.control(minsplit = minsplit, maxdepth = maxdepth), model = TRUE)
+
       #Prediccion
-      prediccion.dt <- dt_prediction(modelo.dt,datos.prueba)
-      updateAceEditor(session, "fieldCodeDtPred", value = codeDtPred(nombreModelo))
-      
+      prediccion.dt <- predict(modelo.dt,datos.prueba)$prediction
+
       #Indices
       indices.dt <- general_indices(datos.prueba[,variable.predecir], prediccion.dt)
-      updateAceEditor(session, "fieldCodeDtIG", value = codeDtIG(variable.predecir))
-      
+
       #isolamos para que no entre en un ciclo en el primer renderPrint
       isolate(modelos$dt[[nombreModelo]] <- list(modelo = modelo.dt, prediccion = prediccion.dt, indices = indices.dt, 
                                                  id = NULL))
+
+
+      isolate({
+        modelos$dt[[nombreModelo]] <- list(modelo = modelo.dt, prediccion = prediccion.dt, indices = indices.dt, 
+                                            id = NULL)
+        modelos2$dt$n <- modelos2$dt$n + 1
+        modelos2$dt$mcs[modelos2$dt$n] <- list(indices.dt)
+        if(modelos2$dt$n > 9)
+          modelos2$dt$n <- 0
+        
+      })
       
-    }, error = function(e){
-      isolate(modelos$dt[[nombreModelo]] <- NULL)
-      showNotification(paste0("Error (DT-00) : ",e), duration = 10, type = "error")
-    },
-    finally = {
-      shinyjs::runjs(code = "generating_model = false")
-    })
-  }
-  
-  
-  #Update model tab
-  output$txtDt <- renderPrint({
-    tryCatch({
       if(!is.null(modelos$dt[[nombreModelo]])){
         modelo.dt <- modelos$dt[[nombreModelo]]$modelo
-        print(modelo.dt)
+        #Cambiamos la forma en que va aparecer el call
+        modelo.dt$call$formula   <- var
+        modelo.dt$call$control$minsplit <- minsplit
+        modelo.dt$call$control$maxdepth <- maxdepth
+        print(summary(modelo.dt))
+        
       }
       else{NULL}
     }, error = function(e){
+      isolate(modelos$dt[[nombreModelo]] <- NULL)
+      
       showNotification(paste0("Error (DT-01) : ",e), duration = 10, type = "error")
       NULL
     })
@@ -165,7 +145,10 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
   output$plot_dt <- renderPlot({
     tryCatch({
       if(!is.null(modelos$dt[[nombreModelo]])){
-        updateAceEditor(session, "fieldCodeDtPlot", value = codeDtPlot(nombreModelo))
+        codigo <- codeDtPlot(nombreModelo)
+        cod    <- paste0("### garbol\n",codigo)
+        
+        isolate(codedioma$code <- append(codedioma$code, cod))
         modelo.dt <- modelos$dt[[nombreModelo]]$modelo
         dt_plot(modelo.dt)
       }
@@ -186,7 +169,7 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
           datos.prueba <- updateData$datos.prueba
           real.val <- datos.prueba[updateData$variable.predecir]
         })
-        tb_predic(real.val, prediccion.dt, updateData$decimals, updateData$idioma)
+        tb_predic(real.val, prediccion.dt, updateData$decimals, codedioma$idioma)
       }
       else{NULL}
       
@@ -207,16 +190,19 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
           variable.predecir <- updateData$variable.predecir
         })
         
-        codigo <- disp_models("prediccion.dt", tr("dt",updateData$idioma), variable.predecir)
-        updateAceEditor(session, "fieldCodeDtDisp", value = codigo)
+        codigo <- disp_models("prediccion.dt", tr("dt",codedioma$idioma), variable.predecir)
+        cod    <- paste0("### docdisp\n",codigo, "\n")
+        
+        isolate(codedioma$code <- append(codedioma$code, cod))
+        
         
         titulos <- c(
-          tr("predvsreal", updateData$idioma),
-          tr("realValue", updateData$idioma),
-          tr("pred", updateData$idioma)
+          tr("predvsreal", codedioma$idioma),
+          tr("realValue", codedioma$idioma),
+          tr("pred", codedioma$idioma)
         )
         
-        plot_real_prediction(datos.prueba[variable.predecir],prediccion.dt,tr("dt", updateData$idioma),titulos)
+        plot_real_prediction(datos.prueba[variable.predecir],prediccion.dt,tr("dt", codedioma$idioma),titulos)
       }
       else{NULL}
     },
@@ -237,8 +223,11 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
         
         modelo.dt <- modelos$dt[[nombreModelo]]$modelo
         
-        updateAceEditor(session, "fieldCodeDtRule", paste0("rpart.rules(model.dt, cover = TRUE,nn = TRUE , style = 'tall', digits=3,
-                            response.name ='",paste0("Rule Number - ", variable.predecir),"')"))
+        codigo <- paste0("rpart.rules(model.dt, cover = TRUE,nn = TRUE , style = 'tall', digits=3,
+                            response.name ='",paste0("Rule Number - ", variable.predecir),"')")
+        cod    <- paste0("### reglas\n",codigo, "\n")
+        
+        isolate(codedioma$code <- append(codedioma$code, cod))
         
         rpart.plot::rpart.rules(modelo.dt, cover = TRUE,nn = TRUE ,roundint=FALSE, style = "tall", digits=3, 
                                 response.name = paste0("Rule Number - ", variable.predecir))
@@ -255,7 +244,7 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
   output$indexdfdt <- renderTable({
     tryCatch({
       if(!is.null(modelos$dt[[nombreModelo]])){
-        idioma <- updateData$idioma
+        idioma <- codedioma$idioma
         indices.dt <- modelos$dt[[nombreModelo]]$indices
         tabla.indicesPrecision(indices.dt, updateData$decimals, idioma)
       }
@@ -271,10 +260,12 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
   
   output$indexdfdt2 <- renderTable({
     tryCatch({
-      if(!is.null(modelos$dt[[nombreModelo]])& !is.null(updateData$summary.var.pred)){
-        idioma <- updateData$idioma
+      if(!is.null(modelos$dt[[nombreModelo]])){
+        idioma <- codedioma$idioma
         decimals <- updateData$decimals
-        tabla.varpred.summary(updateData$summary.var.pred, decimals, idioma)
+        tabla.varpred.summary(summary_indices(updateData$datos.prueba[,updateData$variable.predecir]),
+                              decimals, 
+                              idioma)
       }
       else{NULL}
     }
@@ -285,6 +276,31 @@ mod_regression_trees_server <- function(input, output, session,updateData, model
   },striped = TRUE, bordered = TRUE, spacing = 'l', 
   width = '100%', align = 'c')
   
+  
+  codigo.dt<- function() {
+    isolate({
+      variable.predecir <- updateData$variable.predecir
+      ms <- input$minsplit.dt
+      md <- input$maxdepth.dt
+    })
+    
+    minsplit <- ifelse(!is.numeric(ms), 20, ms)
+    maxdepth <- ifelse(!is.numeric(md), 15, md)
+    
+    #Model generate
+    codigo <- codeDt(variable.predecir,minsplit,maxdepth)
+    cod    <- paste0("### DT\n", codigo)
+    
+    #Prediccion
+    codigo <- codigo.prediccion("dt")
+    cod    <- paste0(cod, codigo)
+    #Indices
+    codigo <- codigo.IG(model.name = "dt", variable.pr = variable.predecir)
+    cod    <- paste0(cod, codigo)
+    
+    isolate(codedioma$code <- append(codedioma$code, cod))
+  }
+  
 }
 
 ## To be copied in the UI
